/*
 * Copyright 2015-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtTest 1.0
import Ubuntu.Components 1.3
import Ubuntu.Test 1.0
import webbrowser.qmltests 0.1
import webbrowserapp.private 0.1

UbuntuTestCase {
    readonly property QtObject app: loader.item

    property var searchEnginesSearchPaths: [
        TestContext.searchEnginesSearchPath,
        Qt.resolvedUrl("../../../src/app/webbrowser/searchengines").toString().substr("file://".length)
    ]

    readonly property string __platformName: TestContext.platformName

    readonly property string baseUrl: "http://localhost:%1/".arg(TestContext.httpServerPort)

    property string historyDbFile: ""
    property string bookmarksDbFile: ""
    property string configFile: ""
    property string searchEngineDescriptionFile: ""
    property string searchEngineDescription: ""

    Loader {
        id: loader
        active: false
        source: "../../../src/app/webbrowser/webbrowser-app.qml"
    }

    SignalSpy {
        id: historyModelSpy
        signalName: "loaded"
    }

    property QtObject base: QtObject {
        function init() {
            if (historyDbFile != "") {
                verify(TestContext.copyDataFile(historyDbFile, "history.sqlite"));
            }
            if (bookmarksDbFile != "") {
                verify(TestContext.copyDataFile(bookmarksDbFile, "bookmarks.sqlite"));
            }
            if (configFile != "") {
                var target = "%1/%2.conf".arg(Qt.application.domain).arg(Qt.application.name);
                verify(TestContext.copyConfigFile(configFile, target));
            }
            if (searchEngineDescriptionFile != "") {
                verify(TestContext.writeDataFile(searchEngineDescription, searchEngineDescriptionFile));
            }

            // Ensure that the singleton models will be reloaded
            // from the per-testcase database files.
            if (HistoryModel.databasePath) {
                historyModelSpy.clear();
                historyModelSpy.target = HistoryModel;
                HistoryModel.databasePath = "";
                historyModelSpy.wait();
                historyModelSpy.target = null;
            }
            BookmarksModel.databasePath = "";
            DownloadsModel.databasePath = "";

            loader.active = true;

            var size = TestContext.initialWindowSize;
            app.initialWindowSize = Qt.size(units.gu(size.width), units.gu(size.height))
        }

        function cleanup() {
            loader.active = false;
            TestContext.removeDataFile("history.sqlite");
            TestContext.removeDataFile("bookmarks.sqlite");
            TestContext.removeDataFile("downloads.sqlite");
            var target = "%1/%2.conf".arg(Qt.application.domain).arg(Qt.application.name);
            TestContext.removeConfigFile(target);
            if (searchEngineDescriptionFile != "") {
                TestContext.removeDataFile(searchEngineDescriptionFile);
            }
        }
    }

    function clickItem(item, button, modifiers) {
        if (button === undefined) button = Qt.LeftButton;
        if (modifiers === undefined) modifiers = Qt.NoModifier;
        var center = centerOf(item);
        mouseClick(item, center.x, center.y, button, modifiers);
    }

    function swipeToDeleteAndConfirm(item) {
        flick(item, item.width / 10, item.height / 2, item.width / 2, 0);
        var confirm = findChild(item, "actionbutton_leadingAction.delete");
        clickItem(confirm);
    }

    function waitFor(predicate, timeout) {
        timeout = timeout || 5000;
        var now = Date.now();
        var end = now + timeout;
        var c = now;
        while (c < end && !predicate()) {
            wait(50);
            c = Date.now();
        }
        return predicate();
    }

    function waitForChild(obj, objectName, timeout) {
        waitFor(function() { return (findChild(obj, objectName) != null); }, timeout);
        return findChild(obj, objectName);
    }
}
