/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "AddressbarBookmark"

    property var bookmarkOptions: null

    function test_switching_tabs_updates_bookmark_toggle() {
        var chrome = findChild(window.contentItem, "chrome");
        bookmarkOptions = bookmark_current_page();
        var dismissButton = findChild(bookmarkOptions, "bookmarkOptions.okButton");
        clickItem(dismissButton);
        tryCompare(this, "bookmarkOptions", null);
        tryCompare(chrome, "bookmarked", true);

        open_new_tab();
        browse_to("test2.html", true);
        check_current_url(baseUrl + "test2.html");
        tryCompare(chrome, "bookmarked", false);

        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_current_url(baseUrl + "test1.html");
        tryCompare(chrome, "bookmarked", true);

        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_current_url(baseUrl + "test2.html");
        tryCompare(chrome, "bookmarked", false);
    }

    function test_cannot_bookmark_empty_page() {
        open_new_tab();
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        check_current_url(baseUrl + "test1.html");
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressbar");
        var bookmarkToggle = findChild(addressbar, "bookmarkToggle");
        tryCompare(bookmarkToggle, "visible", true);
        keyClick(Qt.Key_Tab, Qt.ControlModifier);
        tryCompare(bookmarkToggle, "visible", false);
    }
}
