/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "AddressbarStates"

    function test_cancel_state_loading() {
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressbar");
        browse_to("wait.py?delay=5", false);
        tryCompare(addressbar, "loading", true);
        var actionButton = findChild(addressbar, "actionButton");
        tryCompare(actionButton, "visible", true);
        clickItem(actionButton);
        tryCompare(addressbar, "loading", false);
    }

    function test_state_editing() {
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressbar");
        clickItem(addressbar);
        tryCompare(addressbar, "activeFocus", true);
        keyClick(Qt.Key_Enter);
        tryCompare(addressbar, "activeFocus", false);
    }

    function test_looses_focus_when_loading_starts() {
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressbar");
        clickItem(addressbar);
        tryCompare(addressbar, "activeFocus", true);
        clear_address_bar();
        browse_to("test2.html", false);
        tryCompare(addressbar, "activeFocus", false);
    }

    function test_looses_focus_when_reloading() {
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressbar");
        clickItem(addressbar);
        tryCompare(addressbar, "activeFocus", true);
        var actionButton = findChild(addressbar, "actionButton");
        tryCompare(actionButton, "visible", true);
        // Give some time for the reload action to be ready.
        // XXX: this is racy at best.
        wait(100);
        clickItem(actionButton);
        tryCompare(addressbar, "activeFocus", false);
    }

    function test_clears_when_actual_url_changed() {
        // Regression test for https://launchpad.net/bugs/1456199
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressbar");
        clickItem(addressbar);
        tryCompare(addressbar, "activeFocus", true);
        open_new_tab();
        tryCompare(addressbar, "text", "");
        tryCompare(addressbar, "activeFocus", true);
    }

    function test_does_not_clear_when_typing_while_loading() {
        // Regression test for https://launchpad.net/bugs/1487713
        var chrome = findChild(window.contentItem, "chrome");
        var addressbar = findChild(chrome, "addressbar");
        browse_to("wait.py?delay=3", false);
        tryCompare(addressbar, "loading", true);
        clickItem(addressbar);
        typeString("x");
        tryCompare(addressbar, "loading", false, 10000);
        verify(addressbar.activeFocus);
        compare(addressbar.text, "x");
    }
}
