/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    id: testCase
    name: "BasicAuthentication"

    property var dialog: null

    function request_authentication_dialog() {
        browse_to("basicauth.py", false);
        tryCompare(window.currentWebview, "title", "test page 1");
        dialog = waitForChild(window.contentItem, "httpAuthenticationDialog");
        verify(dialog != null);
    }

    function test_cancel() {
        request_authentication_dialog();
        var denyButton = findChild(dialog, "deny");
        clickItem(denyButton);
        tryCompare(this, "dialog", null);
    }

    function test_input_credentials_data() {
        return [
            {user: "x", password: "x", success: false, title: "test page 1"},
            {user: "user", password: "pass", success: true, title: "AUTH SUCCESS"},
        ];
    }

    function test_input_credentials(data) {
        request_authentication_dialog();
        var usernameField = findChild(dialog, "username");
        clickItem(usernameField);
        tryCompare(usernameField, "activeFocus", true);
        typeString(data.user);
        var passwordField = findChild(dialog, "password");
        clickItem(passwordField);
        tryCompare(passwordField, "activeFocus", true);
        typeString(data.password);
        var allowButton = findChild(dialog, "allow");
        clickItem(allowButton);
        tryCompare(testCase, "dialog", null);
        tryCompare(window.currentWebview, "title", data.title);
        if (!data.success) {
            verify(waitForChild(window.contentItem, "httpAuthenticationDialog"));
        }
    }
}
