/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "ErrorSheet"

    function test_invalid_url_triggers_error_message() {
        var errorSheet = findChild(window.contentItem, "errorSheet");
        verify(!errorSheet.visible);
        browse_to("http://invalid/", true);
        tryCompare(errorSheet, "visible", true);
    }

    function test_navigating_away_discards_error_message() {
        var errorSheet = findChild(window.contentItem, "errorSheet");
        browse_to("http://invalid/", true);
        tryCompare(errorSheet, "visible", true);
        browse_to("test2.html", true);
        tryCompare(errorSheet, "visible", false);
    }

    function test_navigating_back_discards_error_message() {
        var errorSheet = findChild(window.contentItem, "errorSheet");
        browse_to("http://invalid/", true);
        tryCompare(errorSheet, "visible", true);
        var chrome = findChild(window.contentItem, "chrome");
        var backButton = findChild(chrome, "backButton");
        clickItem(backButton);
        tryCompare(window.currentWebview, "url", baseUrl + "test1.html");
        tryCompare(errorSheet, "visible", false);
    }

    function test_navigating_forward_discards_error_message() {
        var errorSheet = findChild(window.contentItem, "errorSheet");
        browse_to("http://invalid/", true);
        tryCompare(errorSheet, "visible", true);
        browse_to("test2.html", true);
        tryCompare(errorSheet, "visible", false);
        var chrome = findChild(window.contentItem, "chrome");
        var backButton = findChild(chrome, "backButton");
        clickItem(backButton);
        tryCompare(errorSheet, "visible", true);
        var forwardButton = findChild(chrome, "forwardButton");
        clickItem(forwardButton);
        tryCompare(errorSheet, "visible", false);
    }
}
