/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtQuick.Window 2.2

SingleWindowTestCase {
    id: testCase
    name: "Fullscreen"

    property var hint: null

    function trigger_page_initiated_fullscreen() {
        browse_to("fullscreen.html", true);
        verify(!window.currentWebview.fullscreen);
        compare(window.visibility, Window.Windowed);
        clickItem(window.currentWebview);
        tryCompare(window.currentWebview, "fullscreen", true);
        tryCompare(window, "visibility", Window.FullScreen);
        hint = waitForChild(window.currentWebview, "fullscreenExitHint");
        verify(hint != null);
        tryCompare(hint, "visible", true);
    }

    function trigger_user_initiated_fullscreen() {
        verify(!window.currentWebview.fullscreen);
        compare(window.visibility, Window.Windowed);
        keyClick(Qt.Key_F11);
        tryCompare(window, "visibility", Window.FullScreen);
        verify(!window.currentWebview.fullscreen);
    }

    function trigger_user_then_page_initiated_fullscreen() {
        browse_to("fullscreen.html", true);
        verify(!window.currentWebview.fullscreen);
        compare(window.visibility, Window.Windowed);
        // User initiated fullscreen
        keyClick(Qt.Key_F11);
        tryCompare(window, "visibility", Window.FullScreen);
        verify(!window.currentWebview.fullscreen);
        // Page initiated fullscreen
        waitForRendering(window.currentWebview);
        clickItem(window.currentWebview);
        compare(window.visibility, Window.FullScreen);
        tryCompare(window.currentWebview, "fullscreen", true);
        hint = waitForChild(window.currentWebview, "fullscreenExitHint");
        verify(hint != null);
        tryCompare(hint, "visible", true);
    }

    function verify_exited_fullscreen() {
        tryCompare(window.currentWebview, "fullscreen", false);
        tryCompare(window, "visibility", Window.Windowed);
        tryCompare(testCase, "hint", null);
    }

    function test_page_initiated_exit() {
        trigger_page_initiated_fullscreen();
        clickItem(window.currentWebview);
        verify_exited_fullscreen();
    }

    function test_page_initiated_user_exit_swipe_up() {
        skip("TO DO: add support for bottom edge drag in tests");
        trigger_page_initiated_fullscreen();
        // TODO: drag bottom edge upwards to open tabs view
        verify_exited_fullscreen();
    }

    function test_page_initiated_user_exit_keypress_data() {
        return [
            { key: Qt.Key_Escape },
            { key: Qt.Key_F11 },
        ];
    }

    function test_page_initiated_user_exit_keypress(data) {
        trigger_page_initiated_fullscreen();
        keyClick(data.key);
        verify_exited_fullscreen();
    }

    function test_user_initiated_user_exit_keypress_data() {
        return test_page_initiated_user_exit_keypress_data();
    }

    function test_user_initiated_user_exit_keypress(data) {
        trigger_user_initiated_fullscreen();
        keyClick(data.key);
        verify_exited_fullscreen();
    }

    function test_user_then_page_initiated_user_exit_keypress_data() {
        return test_page_initiated_user_exit_keypress_data();
    }

    function test_user_then_page_initiated_user_exit_keypress(data) {
        trigger_user_then_page_initiated_fullscreen();
        keyClick(data.key);
        verify_exited_fullscreen();
    }
}
