/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "JSDialogs"

    property var dialog: null

    function test_alert() {
        browse_to("js-alert-dialog.html", false);
        dialog = waitForChild(window.contentItem, "alertDialog");
        compare(dialog.text, "Alert Dialog");
        var okButton = findChild(dialog, "okButton");
        clickItem(okButton);
        tryCompare(this, "dialog", null);
    }

    function test_before_unload_leave() {
        browse_to("js-before-unload-dialog.html", true);
        browse_to("test1.html", false);
        dialog = waitForChild(window.contentItem, "beforeUnloadDialog");
        var leaveButton = findChild(dialog, "leaveButton");
        clickItem(leaveButton);
        tryCompare(this, "dialog", null);
        tryCompare(window.currentWebview, "loading", false);
        compare(window.currentWebview.url, baseUrl + "test1.html");
    }

    function test_before_unload_stay() {
        browse_to("js-before-unload-dialog.html", true);
        browse_to("test1.html", false);
        dialog = waitForChild(window.contentItem, "beforeUnloadDialog");
        var stayButton = findChild(dialog, "stayButton");
        clickItem(stayButton);
        tryCompare(this, "dialog", null);
        tryCompare(window.currentWebview, "loading", false);
        compare(window.currentWebview.url, baseUrl + "js-before-unload-dialog.html");
    }

    function test_confirm_cancel() {
        browse_to("js-confirm-dialog.html", false);
        dialog = waitForChild(window.contentItem, "confirmDialog");
        var cancelButton = findChild(dialog, "cancelButton");
        clickItem(cancelButton);
        tryCompare(this, "dialog", null);
        tryCompare(window.currentWebview, "loading", false);
        compare(window.currentWebview.title, "CANCEL");
    }

    function test_confirm_ok() {
        browse_to("js-confirm-dialog.html", false);
        dialog = waitForChild(window.contentItem, "confirmDialog");
        var okButton = findChild(dialog, "okButton");
        clickItem(okButton);
        tryCompare(this, "dialog", null);
        tryCompare(window.currentWebview, "loading", false);
        compare(window.currentWebview.title, "OK");
    }

    function test_prompt_cancel() {
        browse_to("js-prompt-dialog.html", false);
        dialog = waitForChild(window.contentItem, "promptDialog");
        compare(dialog.text, "Prompt Dialog");
        var textField = findChild(dialog, "inputTextField");
        compare(textField.text, "Default");
        var cancelButton = findChild(dialog, "cancelButton");
        clickItem(cancelButton);
        tryCompare(this, "dialog", null);
        tryCompare(window.currentWebview, "loading", false);
        compare(window.currentWebview.title, "CANCEL");
    }

    function test_prompt_ok() {
        browse_to("js-prompt-dialog.html", false);
        dialog = waitForChild(window.contentItem, "promptDialog");
        compare(dialog.text, "Prompt Dialog");
        var textField = findChild(dialog, "inputTextField");
        compare(textField.text, "Default");
        var text = "TEST";
        clickItem(textField);
        tryCompare(textField, "activeFocus", true);
        var clearButton = findChild(textField, "clear_button");
        clickItem(clearButton);
        typeString(text);
        var okButton = findChild(dialog, "okButton");
        clickItem(okButton);
        tryCompare(this, "dialog", null);
        tryCompare(window.currentWebview, "loading", false);
        compare(window.currentWebview.title, text);
    }
}
