/*
 * Copyright 2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import com.canonical.Oxide 1.0

SingleWindowTestCase {
    id: testCase
    name: "MediaAccess"

    property var dialog: null

    function request_media(audio, video) {
        if (audio) {
            verify(waitFor(function() {
                return Oxide.availableAudioCaptureDevices.length > 0; }));
        }
        if (video) {
            verify(waitFor(function() {
                return Oxide.availableVideoCaptureDevices.length > 0; }));
        }
        browse_to("mediarequest.py?audio=%2&video=%3"
                  .arg(String(audio)).arg(String(video)), true);
    }

    function expect_and_dismiss_dialog(allow) {
        dialog = waitForChild(window.contentItem, "mediaAccessDialog");
        verify(dialog != null);
        var buttonName = allow ? "allowButton" : "denyButton";
        var button = findChild(dialog, "mediaAccessDialog.%1".arg(buttonName));
        clickItem(button);
        tryCompare(testCase, "dialog", null);
        var result = allow ? "allowed" : "denied";
        check_current_url(baseUrl + "media-%1.html".arg(result));
    }

    function test_allow() {
        request_media(true, false);
        expect_and_dismiss_dialog(true);

        // Verify that trying to access the same media for the same origin in
        // the same session will not ask for permission again
        request_media(true, false);
        check_current_url(baseUrl + "media-allowed.html");

        // But it will ask if we try to access other media
        request_media(false, true);
        expect_and_dismiss_dialog(true);

        // Now that we granted both permissions, verify that asking for both
        // together will also not raise the dialog
        request_media(true, true);
        check_current_url(baseUrl + "media-allowed.html");
    }

    function test_deny() {
        // When run separately, all the tests in this TestCase pass, but when
        // run in succession, the first one passes and the others fail. This is
        // because oxide remembers user decisions for the duration of the
        // session for a given host, and there is currently no public API to
        // clear the temporarily saved permissions.
        // See TemporarySavedPermissionContext in Oxide's source code.
        skip("Missing feature in Oxide: https://launchpad.net/+bugs/1495941");

        request_media(true, false);
        expect_and_dismiss_dialog(false);

        // Verify that trying to access the same media for the same origin in
        // the same session will not ask for permission again
        request_media(true, false);
        check_current_url(baseUrl + "media-denied.html");
    }

    function test_deny_combined() {
        skip("Bug in Oxide: https://launchpad.net/bugs/1501017");

        // Deny first one input type, then try to ask both and verify that a
        // request is made for the media that was not asked for the fist time
        request_media(true, false);
        expect_and_dismiss_dialog(false);

        request_media(true, true);
        dialog = waitForChild(window.contentItem, "mediaAccessDialog");
        verify(dialog != null);
    }
}
