/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "MultipleWindows"

    property var window2: null

    function test_open_new_window_data() {
        return [
            {incognito: false, action: "newwindow"},
            {incognito: true, action: "newprivatewindow"},
        ];
    }

    function test_open_new_window(data) {
        compare(app.allWindows.length, 1);
        open_drawer(data.action);
        tryCompare(app.allWindows, "length", 2);
        window2 = app.allWindows[1];
        compare(window2.incognito, data.incognito);
        tryCompare(window2, "visible", true);
        waitFor(function() { return (window2.currentWebview != null); });
        window2.close();
        tryCompare(this, "window2", null);
    }

    function test_open_new_window_progress_bar_hidden() {
        // Regression test for https://launchpad.net/bugs/1638337
        var chrome = findChild(window.contentItem, "chrome");
        var progressBar = findChild(chrome, "chromeProgressBar");
        browse_to("test2.html", true);
        tryCompare(progressBar, "visible", false);
        compare(app.allWindows.length, 1);
        open_drawer("newwindow");
        tryCompare(app.allWindows, "length", 2);
        for (var i in app.allWindows) {
            var w = app.allWindows[i];
            chrome = findChild(w.contentItem, "chrome");
            progressBar = findChild(chrome, "chromeProgressBar");
            tryCompare(progressBar, "visible", false);
        }

        // Clean up
        window = app.allWindows[1];
        window.close();
        tryCompare(this, "window", null);
        window = app.allWindows[0];
    }

    function drag_init() {
        if (!window.wide) {
            skip("Only on wide form factors");
        }
        // Not re-implemented for now as it appears mouseDrag initiates a drag
        // but never ends it. Needs more investigation.
        // https://bazaar.launchpad.net/~phablet-team/webbrowser-app/staging/revision/1579
        skip("TODO: re-implement tab drag tests");
    }

    function test_drag_tab_tabbar_nothing() {
        // Test that dragging a tab out and in of tabbar is a no-op
        drag_init();
        var chrome = findChild(window.contentItem, "chrome");
        var tabsBar = findChild(chrome, "tabsBar");
        var tab = findChild(tabsBar, "tabDelegate");
        var c = centerOf(tab);
        //mouseDrag(tab, c.x, c.y, 0, 50, Qt.LeftButton, Qt.NoModifier, 250);
        //mouseRelease(tab, c.x, c.y + 50);

        /*'''test that dragging a tab out and in of tabbar is same'''
        window = self.app.get_windows()[0]

        # Drag tab down and then back into tab bar
        tab = self.get_tab_delegate(window, 0)
        x2, y2 = self.get_object_center(tab)

        self.drag_tab(tab, x2, y2)

        # Check we still have 1 window and 1 tab
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(1))
        self.assertThat(lambda: len(windows[0].get_webviews()),
                        Eventually(Equals(1)))*/
    }

    function test_drag_tab_bottom_new_window() {
        drag_init();
        /*'''test with two tabs dragging one to the bottom opens a new window'''
        window = self.app.get_windows()[0]

        # Open a new tab and check we have two
        self.open_new_tab()
        self.assertThat(lambda: len(window.get_webviews()),
                        Eventually(Equals(2)))

        # Drag new tab to bottom part of window
        tab = self.get_tab_delegate(window, 1)
        x2, y2 = self.get_object_center(window)

        self.drag_tab(tab, x2, y2)

        # Check that a new window has been opened
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))
        self.assertThat(lambda: len(windows[0].get_webviews()),
                        Eventually(Equals(1)))
        self.assertThat(lambda: len(windows[1].get_webviews()),
                        Eventually(Equals(1)))*/
    }

    function test_drag_tab_outside_new_window() {
        drag_init();
        /*'''test with two tabs dragging one to the bottom opens a new window'''
        window = self.app.get_windows()[0]

        # Open a new tab and check we have two
        self.open_new_tab()
        self.assertThat(lambda: len(window.get_webviews()),
                        Eventually(Equals(2)))

        # Drag new tab outside of window
        tab = self.get_tab_delegate(window, 1)
        x2, y2, width, height = window.globalRect

        if x2 > 20:
            x2 -= 20
        else:
            x2 += width + 20

        self.drag_tab(tab, x2, y2)

        # Check that a new window has been opened
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))
        self.assertThat(lambda: len(windows[0].get_webviews()),
                        Eventually(Equals(1)))
        self.assertThat(lambda: len(windows[1].get_webviews()),
                        Eventually(Equals(1)))*/
    }

    function test_drag_tab_between_windows_move() {
        drag_init();
        /*'''test that dragging a tab from one window to another'''
        # Open a new tab and window
        self.open_new_tab()
        self.open_new_window()

        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))
        self.assertThat(lambda: len(windows[0].get_webviews()),
                        Eventually(Equals(2)))
        self.assertThat(lambda: len(windows[1].get_webviews()),
                        Eventually(Equals(1)))

        # Focus window 0
        self.switch_to_unfocused_window(windows[0])

        # Move tab into window 1
        tab = self.get_tab_delegate(windows[0], 1)
        x2, y2 = self.get_object_center(windows[1].chrome.get_tabs_bar())

        self.drag_tab(tab, x2, y2)

        # Check there are two windows and two tabs open in the second window
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))
        self.assertThat(lambda: len(windows[0].get_webviews()),
                        Eventually(Equals(1)))
        self.assertThat(lambda: len(windows[1].get_webviews()),
                        Eventually(Equals(2)))*/
    }

    function test_drag_tab_between_windows_move_and_close_window() {
        drag_init();
        /*'''test that dragging tab from one window to another closes original'''
        self.open_new_window()
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))

        # Focus window 0
        self.switch_to_unfocused_window(windows[0])

        # Move tab into window 1
        tab = self.get_tab_delegate(windows[0], 0)
        x2, y2 = self.get_object_center(windows[1].chrome.get_tabs_bar())

        self.drag_tab(tab, x2, y2)

        # Check there are two tabs open in the single remaining window
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(1))
        self.assertThat(lambda: len(windows[0].get_webviews()),
                        Eventually(Equals(2)))*/
    }

    function test_drag_public_tab_into_private_window() {
        drag_init();
        /*'''test that you cannot drag a public tab into private window'''
        # Open private window, check there are two windows
        self.open_new_private_window()
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))

        public_window = self.app.get_windows(incognito=False)[0]
        private_window = self.app.get_windows(incognito=True)[0]

        # Focus public window
        self.switch_to_unfocused_window(public_window)

        # Move tab into private window
        tab = self.get_tab_delegate(public_window, 0)
        x2, y2 = self.get_object_center(private_window)

        self.drag_tab(tab, x2, y2)

        # Check there are two windows, one of public and one private
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))
        self.assertThat(len(self.app.get_windows(incognito=False)), Equals(1))
        self.assertThat(len(self.app.get_windows(incognito=True)), Equals(1))*/
    }

    function test_drag_private_tab_into_public_window() {
        drag_init();
        /*'''test that you cannot drag a private tab into public window'''
        # Open private window, check there are two windows
        self.open_new_private_window()
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))

        public_window = self.app.get_windows(incognito=False)[0]
        private_window = self.app.get_windows(incognito=True)[0]

        # Move tab into public window
        tab = self.get_tab_delegate(private_window, 0)
        x2, y2 = self.get_object_center(public_window)

        self.drag_tab(tab, x2, y2)

        # Check there are two windows, one of public and one private
        windows = self.app.get_windows()
        self.assertThat(len(windows), Equals(2))
        self.assertThat(len(self.app.get_windows(incognito=False)), Equals(1))
        self.assertThat(len(self.app.get_windows(incognito=True)), Equals(1))*/
    }
}
