/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "SadTab"

    property var sadTab: null

    function ensure_sad_tab() {
        sadTab = waitForChild(window.contentItem, "sadTab");
        verify(sadTab != null);
        tryCompare(sadTab, "visible", true);
        waitForRendering(sadTab);
    }

    function kill_web_process() {
        var url = window.currentWebview.url;
        window.currentWebview.url = "chrome://kill/"
        // The first time the web process is killed, the browser attempts to
        // reload the page gracefully (after a short delay), hoping the process
        // won’t be killed again.
        wait(500);
        tryCompare(window.currentWebview, "url", url);
        tryCompare(window.currentWebview, "loading", false);
        window.currentWebview.url = "chrome://kill/"
        // The second time around, the browser displays a sad tab.
        ensure_sad_tab();
    }

    function crash_web_process() {
        window.currentWebview.url = "chrome://crash/"
        // A crash of the web process displays the sad tab right away.
        ensure_sad_tab();
    }

    function test_reload_web_process_data() {
        return [
            { executioner: kill_web_process },
            { executioner: crash_web_process },
        ];
    }

    function test_reload_web_process(data) {
        data.executioner();
        var reloadButton = findChild(sadTab, "reloadButton");
        clickItem(reloadButton);
        tryCompare(this, "sadTab", null);
        tryCompare(window.currentWebview, "url", baseUrl + "test1.html");
        tryCompare(window.currentWebview, "loading", false);
    }

    function test_close_tab_web_process_data() {
        return test_reload_web_process_data();
    }

    function test_close_tab_web_process(data) {
        open_new_tab();
        browse_to("test2.html", true);
        tryCompare(window.tabsModel, "count", 2);
        data.executioner();
        var closeTabButton = findChild(sadTab, "closeTabButton");
        clickItem(closeTabButton);
        tryCompare(this, "sadTab", null);
        tryCompare(window.tabsModel, "count", 1);
        check_current_url(baseUrl + "test1.html");
    }
}
