/*
 * Copyright 2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import webbrowser.qmltests 0.1

WebbrowserAppTestCase {
    name: "SitePreviews1"

    property var window: null

    historyDbFile: Qt.resolvedUrl("sitepreviews-history.sqlite")

    function check_capture_exists(capture) {
        return TestContext.checkCacheFileExists("captures/%1".arg(capture));
    }

    function test_cleanup_previews_on_startup() {
        base.init();

        // Initial population of captures directory, before the app is started
        // (thus before PreviewManager.cleanUnusedPreviews() is invoked)
        var capture = Qt.resolvedUrl("capture.png");
        var root = "https://en.wikipedia.org/wiki/";
        var top_countries = ["France", "Japan", "Brazil", "Argentina", "Russia",
                             "Mali", "Canada", "Morocco", "Italy", "Congo"];
        var other_countries = ["Mexico", "China", "Peru"];
        var countries = top_countries.concat(other_countries);
        for (var i in countries) {
            var hash = Qt.md5(root + countries[i]);
            verify(TestContext.copyCacheFile(capture, "captures/%1.png".arg(hash)));
        }
        verify(TestContext.copyCacheFile(capture,
               "captures/%1.png".arg(Qt.md5("http://example.org/"))));
        verify(TestContext.copyCacheFile(Qt.resolvedUrl("notacapture.jpg"),
               "captures/notacapture.jpg"));
        verify(TestContext.copyCacheFile(Qt.resolvedUrl("notanimage.xxx"),
               "captures/notanimage.xxx"));

        // Start the app, which will trigger the asynchronous cleanup of unused
        // captures
        app.init([baseUrl + "test1.html"], true, false);
        tryCompare(app.allWindows, "length", 1);
        window = app.allWindows[0];
        tryCompare(window, "visible", true);
        waitFor(function() { return (window.currentWebview != null); });
        tryCompare(window.currentWebview, "loading", false);

        // Verify that the unused captures are removed
        var removed = [];
        removed.push("notacapture.jpg");
        removed.push("%1.png".arg(Qt.md5("http://example.org/")));
        for (var j in other_countries) {
            removed.push("%1.png".arg(Qt.md5(root + other_countries[j])));
        }
        for (var k in removed) {
            verify(waitFor(
                function() { return !check_capture_exists(removed[k]); }));
        }

        // And that the captures in use are kept
        var kept = [];
        for (var l in top_countries) {
            kept.push("%1.png".arg(Qt.md5(root + top_countries[l])));
        }
        kept.push("%1.png".arg(Qt.md5(baseUrl + "test1.html")));
        for (var m in kept) {
            verify(waitFor(
                function() { return check_capture_exists(kept[m]); }));
        }

        // Cleanup
        window.close();
        tryCompare(this, "window", null);
        base.cleanup();
    }
}
