/*
 * Copyright 2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import webbrowser.qmltests 0.1

SingleWindowTestCase {
    name: "SitePreviews2"

    property var tabsView: null

    configFile: Qt.resolvedUrl("sitepreviews-config.conf")

    function check_capture_exists(capture) {
        return TestContext.checkCacheFileExists("captures/%1".arg(capture));
    }

    function close_tab(index) {
        var count = window.tabsModel.count;
        window.tabsModel.removeTab(index);
        tryCompare(window.tabsModel, "count", count - 1);
    }

    function remove_top_site(url) {
        var newTabView = findChild(window.contentItem, "newTabView");
        verify(newTabView != null);
        var topSites = findChild(newTabView, "topSitesList");
        verify(topSites.visible);
        tryCompare(topSites, "count", 1);
        var topSite = findChild(topSites, "topSiteItem");
        compare(topSite.url, url);
        clickItem(topSite, Qt.RightButton);
        verify(
            waitForChild(window.contentItem, "urlPreviewDelegate.contextMenu"));
        var menu =
            findChild(window.contentItem, "urlPreviewDelegate.contextMenu");
        var deleteAction = findChild(menu, "delete_button");
        clickItem(deleteAction);
        tryCompare(topSites, "count", 0);
    }

    function test_save_on_switch_tab_and_not_delete_if_topsite() {
        // Switching away from tab should save a capture
        open_new_tab();
        var capture = "%1.png".arg(Qt.md5(baseUrl + "test1.html"));
        verify(waitFor(function() { return check_capture_exists(capture); }));

        // Closing the captured tab should not delete the capture since it is
        // now part of the top sites (being the only one we opened so far)
        close_tab(window.wide ? 0 : 1);
        wait(500);
        verify(check_capture_exists(capture));
    }

    function test_save_on_switch_tab_and_delete_if_not_topsite() {
        open_new_tab();
        var url = baseUrl + "test1.html";
        var capture = "%1.png".arg(Qt.md5(url));
        remove_top_site(url);
        close_tab(window.wide ? 0 : 1);
        verify(waitFor(function() { return !check_capture_exists(capture); }));
    }

    function test_delete_when_tab_closed_and_removed_from_topsites() {
        open_new_tab();
        var url = baseUrl + "test1.html";
        var capture = "%1.png".arg(Qt.md5(url));
        close_tab(window.wide ? 0 : 1);
        wait(500);
        verify(check_capture_exists(capture));
        remove_top_site(url);
        verify(waitFor(function() { return !check_capture_exists(capture); }));
    }
}
