/*
 * Copyright 2016-2017 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "Tabs"

    function test_open_target_blank_in_new_tab_data() {
        return [
            { path: "blanktargetlink.html" },
            { path: "fulliframewithblanktargetlink.html" },
        ];
    }

    function test_open_target_blank_in_new_tab(data) {
        browse_to(data.path, true);
        clickItem(window.currentWebview);
        tryCompare(window.tabsModel, "count", 2);
        check_current_url(baseUrl + "test2.html");
        // https://launchpad.net/bugs/1505724
        tryCompare(window.currentWebview, "activeFocus", true);
    }

    // https://launchpad.net/bugs/1464436
    function test_ctrl_click_open_link_in_new_background_tab() {
        browse_to("link.html", true);
        clickItem(window.currentWebview, Qt.LeftButton, Qt.ControlModifier);
        // Eventually we should have two webviews but one should be hidden.
        // Wait some time to increase confidence that webviews won't change
        // their visibility state to an incorrect one before the check.
        tryCompare(window.tabsModel, "count", 2);
        wait(500);
        verify(window.tabsModel.get(0).webview.visible);
        verify(!window.tabsModel.get(1).webview.visible);
        check_current_url(baseUrl + "link.html");
    }

    // https://launchpad.net/bugs/1499780
    function test_ctrl_click_open_link_in_next_tab() {
        open_new_tab();
        window.tabsModel.currentIndex = (window.wide ? 0 : 1);
        browse_to("link.html", true);
        clickItem(window.currentWebview, Qt.LeftButton, Qt.ControlModifier);
        tryCompare(window.tabsModel, "count", 3);
        window.tabsModel.currentIndex = 1;
        check_current_url(baseUrl + "test1.html");
    }

    function test_webview_requests_close() {
        open_new_tab();
        browse_to("closeself.html", true);
        tryCompare(window.tabsModel, "count", 2);
        clickItem(window.currentWebview);
        tryCompare(window.tabsModel, "count", 1);
    }

    function test_last_webview_requests_close() {
        if (window.wide) {
            // XXX: in wide mode, closing the last open tab closes the app,
            // which is not well handled by the test (it segfaults)
            skip("Only on narrow form factors");
        }
        open_new_tab();
        browse_to("closeself.html", true);
        tryCompare(window.tabsModel, "count", 2);
        var tabsView = open_tabs_view();
        var preview = findChild(tabsView, "tab_preview_1");
        var closeButton = findChild(preview, "closeButton");
        clickItem(closeButton);
        var doneButton = findChild(tabsView.parent, "doneButton");
        clickItem(doneButton);
        tryCompare(tabsView, "visible", false);
        tryCompare(window.tabsModel, "count", 1);
        clickItem(window.currentWebview);
        clickItem(window.currentWebview);
        tryCompare(window.tabsModel, "count", 1);
        verify(waitForChild(window.contentItem, "newTabView") != null);
    }
}
