/*
 * Copyright 2016 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4

SingleWindowTestCase {
    name: "TabsView"

    property var tabsView: null
    property var newTabView: null

    function init_open_tabs_view() {
        if (window.wide) {
            skip("Only on narrow form factors");
        }
        tabsView = open_tabs_view();
        verify(tabsView != null);
        tryCompare(tabsView, "visible", true);
    }

    function request_new_tab() {
        var newTabButton = findChild(tabsView.parent, "newTabButton");
        clickItem(newTabButton);
        tryCompare(tabsView, "visible", false);
        newTabView = waitForChild(window.contentItem, "newTabView");
        verify(newTabView != null);
    }

    function close_preview(index) {
        var preview = findChild(tabsView, "tab_preview_%1".arg(index));
        var closeButton = findChild(preview, "closeButton");
        clickItem(closeButton);
    }

    function test_tabs_model() {
        init_open_tabs_view();
        tryCompare(tabsView, "count", 1);
    }

    function test_close_tabs_view(self) {
        init_open_tabs_view();
        var doneButton = findChild(tabsView.parent, "doneButton");
        clickItem(doneButton);
        tryCompare(tabsView, "visible", false);
    }

    function test_open_new_tab() {
        init_open_tabs_view();
        request_new_tab();
        browse_to("test2.html", true);
        tryCompare(this, "newTabView", null);
    }

    function test_close_last_open_tab() {
        init_open_tabs_view();
        close_preview(0);
        tryCompare(tabsView, "visible", false);
        tryCompare(window.tabsModel, "count", 1);
        newTabView = waitForChild(window.contentItem, "newTabView");
        verify(newTabView != null);
        tryCompare(window.currentWebview, "url", "");
    }

    function test_close_current_tab() {
        init_open_tabs_view();
        request_new_tab();
        browse_to("test2.html", true);
        tryCompare(this, "newTabView", null);
        tryCompare(window.tabsModel, "count", 2);
        tabsView = open_tabs_view();
        verify(tabsView != null);
        tryCompare(tabsView, "visible", true);
        tryCompare(tabsView, "count", 2);
        close_preview(0);
        tryCompare(tabsView, "count", 1);
        tryCompare(window.tabsModel, "count", 1);
        var preview = findChild(tabsView, "tab_preview_0");
        compare(preview.title, window.currentWebview.title);
    }

    function test_switch_tabs() {
        init_open_tabs_view();
        check_current_url(baseUrl + "test1.html");
        request_new_tab();
        check_current_url("");
        browse_to("test2.html", true);
        tryCompare(this, "newTabView", null);
        check_current_url(baseUrl + "test2.html");
        for (var i = 1; i <= 2; ++i) {
            tabsView = open_tabs_view();
            verify(tabsView != null);
            tryCompare(tabsView, "visible", true);
            var preview = findChild(tabsView, "tab_preview_1");
            clickItem(preview);
            tryCompare(tabsView, "visible", false);
            check_current_url(baseUrl + "test%1.html".arg(i));
        }
    }

    function test_error_only_for_current_tab() {
        init_open_tabs_view();
        request_new_tab();
        var errorSheet = findChild(window.contentItem, "errorSheet");
        verify(!errorSheet.visible);
        browse_to("http://invalid/", true);
        tryCompare(errorSheet, "visible", true);
        tabsView = open_tabs_view();
        verify(tabsView != null);
        tryCompare(tabsView, "visible", true);
        var preview = findChild(tabsView, "tab_preview_1");
        clickItem(preview);
        tryCompare(tabsView, "visible", false);
        tryCompare(errorSheet, "visible", false);
    }

    function test_selecting_tab_focuses_webview() {
        init_open_tabs_view();
        var preview = findChild(tabsView, "tab_preview_0");
        clickItem(preview);
        tryCompare(tabsView, "visible", false);
        tryCompare(window.currentWebview, "activeFocus", true);
    }

    function test_swipe_partway_switches_tabs() {
        skip("TO DO: add support for bottom edge drag in tests");
        init_open_tabs_view();
        request_new_tab();
        browse_to("test2.html", true);
        check_current_url(baseUrl + "test2.html");
        // TODO: drag_bottom_edge_upwards(0.1)
        check_current_url(baseUrl + "test1.html");
        // TODO: drag_bottom_edge_upwards(0.1)
        check_current_url(baseUrl + "test2.html");
    }
}
